package curve;

import java.awt.geom.Point2D;

/**
 *  An IntersectionFinder determines the point of intersection between
 *  pairs of parametric curves.
 */
public class IntersectionFinder {

  // FIXME - in the future, it may be desirable to support multiple threads.
  // Then theIntersectionFinder can be changed to a thread local object.
  private static IntersectionFinder theIntersectionFinder;
  // execution controls
  private int maxIterations;
  private double tolerance;
  // current curves
  private Curve curve1;
  private Curve curve2;
  // current curve parameters
  private double parameter1;
  private double parameter2;
  // intersection point
  private Point2D intersection;
  // intersection status information
  private int iterationCount;
  private double discriminant;
  private double error;

  /**
   *  IntersectionFinder.createIntersectionFinder() returns the
   *  singleton IntersectionFinder, creating it if necessary.
   * @return 
   */
  public static IntersectionFinder createIntersectionFinder() {
    if (theIntersectionFinder == null) {
      theIntersectionFinder = new IntersectionFinder();
    }
    return theIntersectionFinder;
  }

  /**
   *  new IntersectionFinder() returns a new IntersectionFinder.
   */
  public IntersectionFinder() {
    maxIterations = 10;
    tolerance = 0.01;
  }

  /**
   *  fndr.getTolerance() returns the tolerance for fndr.
   * @return 
   */
  public double getTolerance() {
    return tolerance;
  }

  /**
   *  fndr.setTolerance(tol) sets the tolerance for fndr to tol.
   * @param tol 
   */
  public void setTolerance(double tol) {
    tolerance = tol;
  }

  /**
   *  fndr.getMaxIterations() returns the maximum iteration count
   *  for fndr.
   * @return 
   */
  public int getMaxIterations() {
    return maxIterations;
  }

  /**
   *  fndr.setMaxIterations(max) sets the maximum iteration for
   *  fndr to max.
   * @param max 
   */
  public void setMaxIterations(int max) {
    maxIterations = max;
  }

  /**
   *  fndr.findIntersection(c1, t1, c2, t2) locates an intersection
   *  between curves c1 and c2, if possible.
   *  The intersection search starts with curve parameters t1 for c1
   *  and t2 for c2.
   *  The returned value indicates whether or not an intersection
   *  was found.
   *  If an intersection is found, fndr can be queried for more
   *  information.
   * @param c1 
   * @param t1 
   * @param c2 
   * @param t2 
   * @return 
   */
  public boolean findIntersection(
          Curve c1, double t1,
          Curve c2, double t2) {
    curve1 = c1;
    curve2 = c2;
    parameter1 = t1;
    parameter2 = t2;
    iterationCount = 0;
    intersection = null;
    while (true) {
      double deltaX = curve2.getX(parameter2) - curve1.getX(parameter1);
      double deltaY = curve2.getY(parameter2) - curve1.getY(parameter1);
      error = Math.abs(deltaX) + Math.abs(deltaY);
      if (error < tolerance) {
        return true;
      }
      if (iterationCount >= maxIterations) {
        return false;
      }
      double dx1 = curve1.getDX(parameter1);
      double dy1 = curve1.getDY(parameter1);
      double dx2 = curve2.getDX(parameter2);
      double dy2 = curve2.getDY(parameter2);
      discriminant = dx1 * dy2 - dx2 * dy1;
      parameter1 += (deltaX * dy2 - deltaY * dx2) / discriminant;
      parameter2 += (deltaX * dy1 - deltaY * dx1) / discriminant;
      iterationCount++;
    }
  }

  /**
   *  fndr.getIntersection() returns the intersection found by fndr
   *  when findIntersection() was last invoked.
   * @return 
   */
  public Point2D getIntersection() {
    if (intersection == null) {
      double x = (curve1.getX(parameter1) + curve2.getX(parameter2)) / 2.0;
      double y = (curve1.getY(parameter1) + curve2.getY(parameter2)) / 2.0;
      intersection = new Point2D.Double(x, y);
    }
    return intersection;
  }

  /**
   *  fndr.getParameter1() returns the first curve parameter in the
   *  intersection found by fndr when findIntersection() was last
   *  invoked.
   * @return 
   */
  public double getParameter1() {
    return parameter1;
  }

  /**
   *  fndr.getParameter2() returns the second curve parameter in the
   *  intersection found by fndr when findIntersection() was last
   *  invoked.
   * @return 
   */
  public double getParameter2() {
    return parameter2;
  }

  /**
   *  fndr.getDiscriminant() returns the last value of the
   *  discriminant when findIntersection() was last invoked.
   * @return 
   */
  public double getDiscriminant() {
    return discriminant;
  }

  /**
   *  fndr.getIterationCount() returns the number of iterations used
   *  when findIntersection() was last invoked.
   * @return 
   */
  public int getIterationCount() {
    return iterationCount;
  }

  /**
   *  fndr.getError() returns the error in the intersection found by
   *  fndr when findIntersection() was last invoked.
   *  The error is the distance between the point on the first curve
   *  at parameter getParameter1() and the point on the second curve
   *  at parameter getParameter2().
   * @return 
   */
  public double getError() {
    return error;
  }

}  // public class IntersectionFinder

