package display;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.IOException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * The ComponentGraphics class provides services for printing or exporting an
 * image created from a Swing JComponent.
 * Usually the best way to use this class is to invoke one of the static methods
 * getExportAction() or getPrintAction().
 */
public class ComponentGraphics {

  private JComponent component;

  private Action printAction;
  private PrinterJob printerJob;
  private Printable printable;

  private Action exportAction;
  private ImageFileChooser chooser;

  /**
   * startPrint() displays a print dialog and initiates printing.
   */
  private void startPrint() {
    if (printerJob == null) {
      printerJob = PrinterJob.getPrinterJob();
      printerJob.setPrintable(printable);
    }
    if (printerJob.getPrintService() == null) {
      printerJob = PrinterJob.getPrinterJob();
      if (printerJob.getPrintService() == null) {
        JOptionPane.showMessageDialog(component,
                "No print service available",
                "Print Error", JOptionPane.ERROR_MESSAGE);
      }
      return;
    }
    if (printerJob.printDialog()) {
      try {
        printerJob.print();
      } catch (PrinterException e) {
        JOptionPane.showMessageDialog(component, e.toString(),
                "Print Error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  /**
   * startExport() displays an export dialog and initiates exporting to a file.
   */
  private void startExport() {
    int result = chooser.showDialog();
    if (result == JFileChooser.APPROVE_OPTION) {
      try {
        File file = chooser.getSelectedFile();
        ImageType type = chooser.getFileType();
        type.write(file, component);
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(component, ex.getMessage(),
                "Export Error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  /**
   * new ComponentGraphics(c) returns a new ComponentGraphics for exporting c.
   * 
   * @param c the component to be made imageable
   */
  public ComponentGraphics(JComponent c) {
    component = c;
    printAction = null;
    printerJob = null;
    printable = new Printable() {

      public int print(Graphics g, PageFormat fmt, int pgnum) {
        if (pgnum > 0) {
          return Printable.NO_SUCH_PAGE;
        }
        Graphics2D g2d = (Graphics2D) g;
        g2d.translate(fmt.getImageableX(), fmt.getImageableY());
        double xScale = fmt.getImageableWidth() / component.getWidth();
        double yScale = fmt.getImageableHeight() / component.getHeight();
        double scale = Math.min(xScale, yScale);
        g2d.scale(scale, scale);
        component.print(g);
        return Printable.PAGE_EXISTS;
      }

    };
    exportAction = null;
    chooser = new ImageFileChooser();
  }

  /**
   * cp.getPrintAction() returns an printAction that initiates printing of cp.
   *
   * @return the print action
   */
  public Action getPrintAction() {
    if (printAction == null) {
      printAction = new AbstractAction() {

        public void actionPerformed(ActionEvent e) {
          startPrint();
        }

      };
      printAction.putValue(Action.NAME, "Print...");
    }
    return printAction;
  }

  /**
   * cg.getExportAction() returns an export action that initiates graphics
   * export of cg.
   * 
   * @return the export action
   */
  public Action getExportAction() {
    if (exportAction == null) {
      exportAction = new AbstractAction() {

        public void actionPerformed(ActionEvent e) {
          startExport();
        }

      };
      exportAction.putValue(Action.NAME, "Export...");
    }
    return exportAction;
  }

  /**
   * ComponentGraphics.getPrintAction(JComponent c) creates a ComponentGraphics
   * for c and returns its print action.
   *
   * @param c the component to be made printable
   * @return the print action for c
   */
  public static Action getPrintAction(JComponent c) {
    ComponentGraphics pc = new ComponentGraphics(c);
    return pc.getPrintAction();
  }

  /**
   * ComponentGraphics.getExportAction(JComponent c) creates a ComponentGraphics
   * for c and returns its export action.
   * 
   * @param c the component to be made exportable
   * @return the export action for c
   */
  public static Action getExportAction(JComponent c) {
    ComponentGraphics pc = new ComponentGraphics(c);
    return pc.getExportAction();
  }

}  // public class ComponentPrintable
