package display;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;

/**
 * A Model represents a visual data structure on which operations can be
 * performed.
 * In addition to the methods in this interface, a model normally helper methods
 * for its operations and other .
 */
public class OperationController {

  // Singleton instance
  private static OperationController controller;
  // Instance variables
  private Thread operationThread;
  private List<Action> actions;
  private Map<Model, PaintContext> paintContexts;
  private Action stopOperationAction;
  private Action showChangesAction;
  private Action initializeAction;
  private Action runAction;
  private Action stepAction;
  private boolean running;
  private boolean pause;
  private Log log;

  private void enableActions(boolean enabled) {
    for (Action action : actions) {
      action.setEnabled(enabled);
    }
    if (stopOperationAction != null) {
      stopOperationAction.setEnabled(!enabled);
    }
  }

  private synchronized void setOperationThread(Thread operationThread) {
    this.operationThread = operationThread;
  }

  private synchronized boolean isOperationThread() {
    return Thread.currentThread() == operationThread;
  }

  private synchronized boolean isOperationRunning() {
    return operationThread != null;
  }

  private PaintContext createPaintContext(Grid grid) {
    return new PaintContext(grid);
  }

  private synchronized void setRun() {
    this.running = true;
    this.pause = false;
    notifyAll();
  }

  private synchronized void setStep() {
    this.pause = running;
    running = false;
    notifyAll();
  }

  private synchronized void checkPause() {
    if (running) {
      return;
    }
    while (pause) {
      try {
        wait();
      } catch (InterruptedException exception) {
        System.out.println(exception.getMessage());
      }
    }
    pause = true;
  }

  private OperationController() {
    actions = new ArrayList<Action>();
    paintContexts = new HashMap<Model, PaintContext>();
    pause = false;
    running = true;
  }

  public void setContext(Model model, PaintContext context) {
    paintContexts.put(model, context);
  }

  public PaintContext getContext(Model model) {
    return paintContexts.get(model);
  }

  public static OperationController createOperationController() {
    if (controller == null) {
      controller = new OperationController();
    }
    return controller;
  }

  /**
   * DOCME
   * @param operation
   * @return
   */
  public <T extends Model> Action createOperationAction(final T model,
          final Operation<T> operation) {
    Action action = new AbstractAction(operation.toString()) {

      public void actionPerformed(ActionEvent event) {
        if (isOperationRunning()) {
          System.out.println("Another operation is running");
          return;
        }
        final Thread newThread = new Thread() {

          @Override
          public void run() {
            setOperationThread(Thread.currentThread());
            model.resetView();
            paintContexts.get(model).show();
            getLog().clearText();
            getLog().addText("Executing " + operation + ".");
            operation.operate(model);
            paintContexts.get(model).show();
            getLog().addText("Operation complete.");
            setOperationThread(null);
            enableActions(true);
          }

        };
        enableActions(false);
        newThread.setDaemon(false);
        newThread.start();
      }

    };
    actions.add(action);
    return action;
  }

  /**
   * DOCME
   * @return
   */
  public Action createStopOperationAction() {
    if (stopOperationAction == null) {
      stopOperationAction = new AbstractAction("Stop Operation") {

        public void actionPerformed(ActionEvent event) {
          setOperationThread(null);
          enableActions(true);
          getLog().addText("Operation stopped.");
        }

      };
    }
    stopOperationAction.setEnabled(false);
    return stopOperationAction;
  }

  /**
   * DOCME
   * @return
   */
  public Action createShowChangesAction() {
    if (showChangesAction == null) {
      showChangesAction = new AbstractAction("Show Changes") {

        public void actionPerformed(ActionEvent event) {
          AbstractButton button = (AbstractButton) event.getSource();
          boolean isSelected = button.isSelected();
          for (PaintContext context : paintContexts.values()) {
            context.setShowChanges(isSelected);
          }
          runAction.setEnabled(isSelected);
          stepAction.setEnabled(isSelected);
          if (!isSelected) {
            setRun();
          }
        }

      };
      showChangesAction.setEnabled(true);
    }
    return showChangesAction;
  }

  /**
   * DOCME
   * @return
   */
  public <T extends Model> Action createInitializeAction(final T model) {
    if (initializeAction == null) {
      initializeAction = new AbstractAction("Initialize") {

        public void actionPerformed(ActionEvent event) {
          model.resetView();
          paintContexts.get(model).show();
        }

      };
      actions.add(initializeAction);
    }
    return initializeAction;
  }

  public void showChanges() {
    if (!isOperationThread()) {
      throw new IllegalStateException("Not operation thread");
    }
    for (PaintContext context : paintContexts.values()) {
      context.showChanges();
    }
    checkPause();
  }

  /**
   * DOCME
   * @return
   */
  public Action createRunAction() {
    if (runAction == null) {
      runAction = new AbstractAction("Run") {

        public void actionPerformed(ActionEvent event) {
          setRun();
        }

      };
      runAction.setEnabled(!running);
    }
    return runAction;
  }

  /**
   * DOCME
   * @return
   */
  public Action createStepAction() {
    if (stepAction == null) {
      stepAction = new AbstractAction("Step") {

        public void actionPerformed(ActionEvent event) {
          setStep();
        }

      };
      stepAction.setEnabled(!running);
    }
    return stepAction;
  }

  public Log getLog() {
    if (log == null) {
      log = new Log();
    }
    return log;
  }

}
