package graph;

import java.awt.Point;
import java.util.Comparator;
import java.util.Iterator;

/**
 * A Graph provides a view of objects as either vertices or edges in a graph.
 * It provides information about the vertices and edges needed by a
 * GraphSearcher to perform a graph search.
 * An implementation of the graph interface may throw an
 * IllegalArgumentException for any method parameters that it cannot interpret
 * as a vertex or edge in its view of objects or when its interpretation of an
 * object is of the wrong type (e.g. a vertex when the method expects an edge).
 *
 */
public abstract class Graph
        implements Iterable<Object> {

  /**
   * g.getUndirected() returns true if the edges in g are undirected edges.
   * @return true if the edges are undirected edges
   */
  public abstract boolean isUndirected();

  /**
   * g.iterator() returns an iterator for the vertices in g.
   * This implements the java.lang.Iterable<Object> interface.
   * All of the vertices in g can be processed with the following loop:
   * <p>
   * for (Object v : g) { code to process vertex v }
   * </p>
   * @return an iterator for the vertices
   */
  public abstract Iterator<Object> iterator();

  /**
   * g.getEdgesFrom(v) returns an iterable for the edges leaving from vertex v
   * in g.
   * @param v a vertex
   * @return an iterator for the edges leaving v
   */
  public abstract Iterable<Object> getEdgesFrom(Object v);

  /**
   * g.getPosition(v) returns the position of vertex v in g.
   * @param v a vertex
   * @return its position
   */
  public abstract Point getPosition(Object v);

  /**
   * g.getToVertex(e) returns the from vertex for the edge e in g.
   * @param e an edge
   * @return its from vertex (a.k.a. source, tail)
   */
  public abstract Object getFromVertex(Object e);

  /**
   * g.getToVertex(e) returns the to vertex for the edge e in g.
   * @param e an edge
   * @return its to vertex (a.k.a. destination, head)
   */
  public abstract Object getToVertex(Object e);

  /**
   * g.getWeight(e) returns the weight for the edge e in g.
   * If the weight is not defined NaN is returned.
   * @param e an edge
   * @return its weight
   */
  public abstract double getWeight(Object e);

  /**
   * g.getWeight(e) returns the opposite edge for the edge e in g.
   * If g.isUndirected() is false then null is returned for all edges.
   * @param e an edge
   * @return it weight
   */
  public abstract Object getOpposite(Object e);

  /**
   * g.search(startVertex, visitor) escorts visitor through a search of the
   * current graph of g using the current dispenser of g.
   */
  public void search(Object startVertex, GraphVisitor visitor, Dispenser dispenser) {
    // FIXME - add code to perform a graph search, invoking visitor methods at
    // appropriate times:
    // visitor.isVisited(): determine if a vertex has been visited.
    // visitor.visitVertex(): inform the visitor that a vertex has been visited.
    // visitor.enqueueEdge(): inform the visitor that an edge has been enqueued.
    // visitor.visitEdge(): inform the visitor that an edge has been visited.
    // visitor.discardEdge(): inform the visitor that an edge has been discarded
    // (it just came off the dispenser but its to vertex was already visited).
    // Do not invoke GraphModel methods.
    //
    // For accessing edges leaving a vertex or the to and from vertex of an
    // edge, send messages to the graph instance variable of this class.
    visitor.visitVertex(startVertex);
    for (Object edge : getEdgesFrom(startVertex)) {
      dispenser.add(edge);
      visitor.enqueueEdge(edge);
    }
    while (!dispenser.isEmpty()) {
      Object edge = dispenser.remove();
      Object toVertex = getToVertex(edge);
      if (!visitor.isVisited(toVertex)) {
        visitor.visitEdge(edge);
        visitor.visitVertex(toVertex);
        for (Object newEdge : getEdgesFrom(toVertex)) {
          dispenser.add(newEdge);
          visitor.enqueueEdge(newEdge);
        }
      } else {
        visitor.discardEdge(edge);
      }
    }
  }

  public Comparator<Object> getEdgeComparator() {
    return new Comparator<Object>() {

      public int compare(Object e1, Object e2) {
        double difference = getWeight(e1) - getWeight(e2);
        if (difference < 0) {
          return -1;
        }
        if (difference > 0) {
          return 1;
        }
        return 0;
      }

    };
  }

}
