package graph;

import display.ArcVisual;
import display.Grid;
import display.LinearMover;
import display.Model;
import display.Mover;
import display.OperationController;
import display.PaintContext;
import display.PainterVisual;
import display.PrimaryVisual;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.util.HashMap;
import java.util.Map;
import painter.EllipsePainter;
import painter.ShapePainter;
import painter.TextPainter;

/**
 *
 */
public class GraphModel
        implements Model {

  private ShapePainter vertexPainter;
  private Mover mover;

  private class VertexData {

    private PainterVisual vertexVisual;
    private PainterVisual labelVisual;
    private VertexState state;

    private VertexData(Graph graph, Object vertex, Font font) {
      state = VertexState.unvisited;
      vertexVisual = new PainterVisual(vertexPainter, mover);
      vertexVisual.setColor(Color.black);
      vertexVisual.setFillColor(state.getColor());
      Point position = graph.getPosition(vertex);
      vertexVisual.setPosition(position.x, position.y);
      vertexVisual.setZ(2);
      vertexVisual.commitChanges();
      String label = vertex.toString();
      TextPainter labelPainter = new TextPainter(label, font, 0);
      labelVisual = new PainterVisual(labelPainter, mover);
      labelVisual.setColor(Color.blue);
      labelVisual.setPosition(position.x, position.y);
      labelVisual.setZ(2);
      labelVisual.commitChanges();
    }

    public boolean setState(VertexState state) {
      if (state != null) {
        this.state = state;
        vertexVisual.setFillColor(state.getColor());
        return true;
      }
      return false;
    }

  }

  public class EdgeData {

    private static final float labelFontSize = 16;
    private ArcVisual edgeVisual;
    private EdgeState state;
    private Object dominantEdge;

    public EdgeData(Object edge) {
      String from = graph.getFromVertex(edge).toString();
      String to = graph.getToVertex(edge).toString();
      PrimaryVisual v1 = vertexDatas.get(from).vertexVisual;
      PrimaryVisual v2 = vertexDatas.get(to).vertexVisual;
      edgeVisual = new ArcVisual(v1, v2, labelFontSize);
      edgeVisual.setStrokeWidth(3);
      edgeVisual.setColor(EdgeState.start.getColor());
      edgeVisual.setZ(0);
      double weight = graph.getWeight(edge);
      if (weight != Double.NaN) {
        edgeVisual.setLabel(numberString(weight));
      }
      edgeVisual.commitChanges();
      dominantEdge = edge;
    }

    public EdgeData(ArcVisual visual, EdgeState state) {
      this.edgeVisual = visual;
      this.state = state;
    }

    public boolean setState(EdgeState state, Object edge) {
      if (state != null) {
        if (state == EdgeState.enqueued) {
          String to = graph.getToVertex(edge).toString();
          edgeVisual.setTo(vertexDatas.get(to).vertexVisual);
          dominantEdge = edge;
        }
        if (state == EdgeState.start || edge == dominantEdge) {
          this.state = state;
          edgeVisual.setColor(state.getColor());
        }
        return true;
      }
      return false;
    }

  }

  private PaintContext context;
  private OperationController controller;
  private Map<Object, VertexData> vertexDatas;
  private Map<Object, EdgeData> edgeDatas;
  private Graph graph;

  /**
   * DOCME
   */
  private class SearchVisualizer
          extends GraphVisitor {

    private GraphVisitor visitor;

    private SearchVisualizer() {
      this(null);
    }

    private SearchVisualizer(GraphVisitor visitor) {
      this.visitor = visitor;
    }

    @Override
    public void enqueueEdge(Object edge) {
      if (visitor != null) {
        visitor.enqueueEdge(edge);
      }
      EdgeData edgeData = edgeDatas.get(edge);
      if (edgeData.setState(edgeData.state.enqueue(), edge)) {
        if (visitor != null && visitor instanceof PrioritizingGraphVisitor) {
          PrioritizingGraphVisitor prioritizingVisitor =
                  (PrioritizingGraphVisitor) visitor;
          double priority = prioritizingVisitor.getEdgePriority(edge);
          edgeData.edgeVisual.setLabel(numberString(priority));
        }
        controller.showChanges();
      }
    }

    @Override
    public void visitEdge(Object edge) {
      if (visitor != null) {
        visitor.visitEdge(edge);
      }
      EdgeData edgeData = edgeDatas.get(edge);
      if (edgeData.setState(edgeData.state.visit(), edge)) {
        controller.showChanges();
      }
    }

    @Override
    public void discardEdge(Object edge) {
      if (visitor != null) {
        visitor.discardEdge(edge);
      }
      EdgeData edgeData = edgeDatas.get(edge);
      if (edgeData.setState(edgeData.state.discard(), edge)) {
        controller.showChanges();
      }
    }

    @Override
    public boolean visitVertex(Object vertex) {
      if (visitor != null) {
        visitor.visitVertex(vertex);
      }
      VertexData vertexData = vertexDatas.get(vertex);
      if (vertexData.setState(vertexData.state.visit())) {
        controller.showChanges();
      }
      return true;
    }

    public boolean isVisited(Object vertex) {
      if (visitor != null) {
        return visitor.isVisited(vertex);
      }
      return vertexDatas.get(vertex).state == VertexState.visited;
    }

  }

  private String numberString(double number) {
    if (Integer.MIN_VALUE <= number && number <= Integer.MAX_VALUE) {
      int intValue = (int) Math.round(number);
      double error = Math.abs(number - intValue);
      if (error*1000 < Math.abs(number)) {
        return Integer.toString(intValue);
      }
    }
    return Double.toString(number);
  }

  private void addVertex(Object vertex) {
    Grid grid = context.getGrid();
    Font font = grid.getFont();
    VertexData vertexData = new VertexData(graph, vertex, font);
    vertexDatas.put(vertex, vertexData);
    context.addVisual(vertexData.vertexVisual);
    context.addVisual(vertexData.labelVisual);
  }

  private void addEdge(Object edge) {
    // If edgeVisuals already contains the edge then the graph is undirected and
    // the EdgeData object was already created when the opposite edge was added.
    if (edgeDatas.containsKey(edge)) {
      return;
    }
    EdgeData edgeData = new EdgeData(edge);
    context.addVisual(edgeData.edgeVisual);
    edgeDatas.put(edge, edgeData);
    // The two directed edges for an undirected edge share an EdgeData object.
    // The shared object is created when the first of the two directed edges
    // is added.
    if (graph.isUndirected()) {
      Object oppositeEdge = graph.getOpposite(edge);
      edgeDatas.put(oppositeEdge, edgeData);
    }
  }

  private Grid createGrid() {
    Font font = new Font(Font.SERIF, Font.PLAIN, 20);
    Grid grid = new Grid(font, 7, 5);
    grid.setPads(0.5, 0.5);
    grid.setSteps(4.0, 4.0);
    vertexPainter = new EllipsePainter(40, 40, 3);
    mover = new LinearMover();
    vertexDatas = new HashMap<Object, VertexData>();
    edgeDatas = new HashMap<Object, EdgeData>();
    return grid;
  }

  public void resetView() {
    reset();
  }

  /**
   * DOCME
   */
  public GraphModel() {
    super();
    controller = OperationController.createOperationController();
    context = new PaintContext(createGrid());
    controller.setContext(this, context);
  }

  public void reset() {
    if (graph == null) {
      return;
    }
    for (VertexData vertexData : vertexDatas.values()) {
      vertexData.setState(VertexState.unvisited);
    }
    for (EdgeData edgeData : edgeDatas.values()) {
      edgeData.setState(EdgeState.start, null);
    }
  }

  public void setGraph(Graph graph) {
    context.clear();
    context.show();
    this.graph = graph;
    vertexDatas.clear();
    edgeDatas.clear();
    for (Object vertex : graph) {
      addVertex(vertex);
    }
    for (Object vertex : graph) {
      for (Object edge : graph.getEdgesFrom(vertex)) {
        addEdge(edge);
      }
    }
    context.show();
  }

  public Graph getGraph() {
    return graph;
  }

  /**
   * gm.getSearchVisualizer() returns a graph visitor that displays an animation
   * for a graph search.
   * The search does nothing except for the animation.
   * @return a search visualizer
   */
  public GraphVisitor getSearchVisualizer() {
    return new SearchVisualizer();
  }

  /**
   * gm.getSearchVisualizer(visitor) returns a graph visitor that displays an
   * animation for a graph search.
   * The search simultaneously invokes methods of visitor.
   * @return a search visualizer
   */
  public GraphVisitor getSearchVisualizer(GraphVisitor visitor) {
    return new SearchVisualizer(visitor);
  }

}
