package graph;

import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * DOCME
 */
public abstract class SimpleGraph
        extends Graph {

  private class EdgeIterable
          implements Iterable<Object> {

    private List<Object> edges;

    private EdgeIterable() {
      edges = new ArrayList<Object>();
    }

    private void addEdge(Object edge) {
      edges.add(edge);
    }

    public Iterator<Object> iterator() {
      return edges.iterator();
    }

  }

  private List<Object> vertices;
  private Map<Object, EdgeIterable> edgesFrom;
  private Map<Object, Point> positions;
  private Map<Object, Object> fromVertices;
  private Map<Object, Object> toVertices;
  private Map<Object, Double> weights;
  private Map<Object, Object> opposites;
  private boolean undirected;

  protected abstract void buildGraph();

  protected void setUndirected(boolean undirected) {
    if (vertices.size() > 0) {
      throw new IllegalStateException("setUndirected() cannot be called " +
              "after vertices have been added");
    }
    this.undirected = undirected;
  }

  private String addDirectedEdge(String from, String to) {
    return addDirectedEdge(from, to, Double.NaN);
  }

  private String addDirectedEdge(String from, String to, double weight) {
    if (!vertices.contains(from)) {
      throw new IllegalArgumentException("From vertex " + from + " not found");
    }
    if (!vertices.contains(to)) {
      throw new IllegalArgumentException("To vertex " + to + " not found");
    }
    String newEdge = from + "->" + to;
    EdgeIterable edgeIterable = edgesFrom.get(from);
    edgeIterable.addEdge(newEdge);
    fromVertices.put(newEdge, from);
    toVertices.put(newEdge, to);
    weights.put(newEdge, weight);
    return newEdge;
  }

  public SimpleGraph() {
    vertices = new ArrayList<Object>();
    edgesFrom = new HashMap<Object, EdgeIterable>();
    positions = new HashMap<Object, Point>();
    fromVertices = new HashMap<Object, Object>();
    toVertices = new HashMap<Object, Object>();
    weights = new HashMap<Object, Double>();
    opposites = new HashMap<Object, Object>();
    undirected = true;
    buildGraph();
  }

  /**
   * g.getUndirected() returns true if the edges in g are undirected edges.
   * @return true if the edges are undirected edges
   */
  public boolean isUndirected() {
    return undirected;
  }

  public void addVertex(String v, int x, int y) {
    if (vertices.contains(v)) {
      throw new IllegalArgumentException("Vertex " + v + " already added");
    }
    vertices.add(v);
    edgesFrom.put(v, new EdgeIterable());
    positions.put(v, new Point(x, y));
  }

  public void addEdge(String from, String to) {
    addEdge(from, to, Double.NaN);
  }

  public void addEdge(String from, String to, double w) {
    String forward = addDirectedEdge(from, to, w);
    if (undirected) {
      String reverse = addDirectedEdge(to, from, w);
      opposites.put(forward, reverse);
      opposites.put(reverse, forward);
    }
  }

  /**
   * g.iterator() returns an iterator for the vertices in g.
   * This implements the java.lang.Iterable<Object> interface.
   * All of the vertices in g can be processed with the following loop:
   * <p>
   * for (Object v : g) { code to process vertex v }
   * </p>
   * @return an iterator for the vertices
   */
  public Iterator<Object> iterator() {
    return vertices.iterator();
  }

  /**
   * g.getEdgesFrom(v) returns an iterator for the edges leaving from vertex v
   * in g.
   * @param v a vertex
   * @return an iterator for the edges leaving v
   */
  public Iterable<Object> getEdgesFrom(Object v) {
    return edgesFrom.get(v);
  }

  /**
   * g.getPosition(v) returns the position of vertex v in g.
   * @param v a vertex
   * @return the position of v
   */
  public Point getPosition(Object v) {
    return positions.get(v);
  }

  /**
   * g.getToVertex(e) returns the from vertex for the edge e in g.
   * @param e an edge
   * @return its from vertex (a.k.a. source, tail)
   */
  public Object getFromVertex(Object e) {
    return fromVertices.get(e);
  }

  /**
   * g.getToVertex(e) returns the to vertex for the edge e in g.
   * @param e an edge
   * @return its to vertex (a.k.a. destination, head)
   */
  public Object getToVertex(Object e) {
    return toVertices.get(e);
  }

  /**
   * g.getWeight(e) returns the weight for the edge e in g.
   * If the weight is not defined NaN is returned.
   * @param e an edge
   * @return its weight
   */
  public double getWeight(Object e) {
    return weights.get(e);
  }

  /**
   * g.getWeight(e) returns the opposite edge for the edge e in g.
   * If g.isUndirected() is false then null is returned for all edges.
   * @param e an edge
   * @return its opposite edge
   */
  public Object getOpposite(Object e) {
    return opposites.get(e);
  }

}
